\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("a349009-HAf.gp");
read("my-oeis.gp");

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ A349009 HAf decimal digits

{
  my(want=[
 9,7,6,1,6,4,0,0,2,9,1,2,7,0,3,5,1,3,4,0,6,4,0,7,1,5,8,0,8,4,2,1,1,1,2,
 9,7,2,6,3,1,2,1,9,9,3,1,7,3,2,6,9,0,5,2,4,3,4,9,4,8,8,0,3,0,0,8,2,8,7,
 3,8,6,7,9,6,5,1,1,6,0,1,1,0,7,5,0,4,2,4,7,8,8,5,1,6,1,5,8,6,3,8,6,6,9
]);
  check_equal(a_vector(#want), want, "a_vector() vs DATA");
}

{
  check_equal(vector(20,len,len--; #a_vector(len)),
              vector(20,len,len--; len));

  my(prev=[]);
  for(len=1,500,
     my(v=a_vector(len));
     check_equal(v[1..#prev], prev);
     prev=v);
}


\\------------------------------------------------------------------------------

A070402(n) = [1,2,4,3][n%4+1];
{
  my(v=OEIS_samples("A070402"));  \\ OFFSET=0
  check_equal(vector(#v,n,n--; A070402(n)), v);
}

{
  check_equal(vector(10,n, real(b^n) % 5),
              vector(10,n, A070402(n-1)),
              "b^n real");

  check_equal(vector(10,n, imag(b^n) % 5),
              vector(10,n, A070402(n)),
              "b^n imag");
}


\\------------------------------------------------------------------------------
check_equal(d,'d);
check_equal(k,'k);
check_equal(m,'m);
check_equal(n,'n);
check_equal(n,'n);
check_equal(p,'p);
check_equal(r,'r);
check_equal(v,'v, "v not a global");
check_equal(HAf,'HAf);
check_equal(ret,'ret);

\\------------------------------------------------------------------------------
print("end");





\\ Unused:
\\
\\ \\-----------------------------------------------------------------------------
\\ \\ RotQ Quadrants
\\ 
\\ \\ In the code, RotQ(z) pays no particular attention to whether z on the
\\ \\ real or imaginary axes is reckoned quadrants 1 and 3 already, or rotate.
\\ \\ This is since the uses by HAgrowf mean z is never on the axes, and even
\\ \\ if it was then the result is HAgrowf(z) = 0 for all the axes.
\\ 
\\ \\ HAgrowf=0 for z on all axes
\\ HAgrowf( 10  ) == 0   || error();
\\ HAgrowf(-10  ) == 0   || error();
\\ HAgrowf( 10*I) == 0   || error();
\\ HAgrowf(-10*I) == 0   || error();
\\ 
\\ \\ z = 1/b^j not on the real or imaginary axes follows by noting the real
\\ \\ and imaginary parts of b^j are a repeating pattern != 0 mod 5.
\\ {
\\   for(j=1,100,
\\      real(b^j) % 5 == [3,1,2,4][j%4+1]  || error();
\\      imag(b^j) % 5 == [1,2,4,3][j%4+1]  || error();
\\      my(z=1/b^j);  (real(z) != 0 && imag(z) != 0)   || error());
\\ }
\\ \\ Factor 4+I changes the pattern of remainders, but all still != 0.
\\ {
\\   (4+I)/b == 6/5 - 7/5*I   || error();
\\   for(j=2,100,
\\      real((4+I)*b^j) % 5 == [1,2,4,3][j%4+1]  || error();
\\      imag((4+I)*b^j) % 5 == [2,4,3,1][j%4+1]  || error();
\\      my(z=(4+I)/b^j);  (real(z) != 0 && imag(z) != 0)   || error());
\\ }
\\ 
\\ 
\\ \\-----------------------------------------------------------------------------
\\ \\ Upper Bound Test
\\ 
\\ \\ The omitted = 1/sqrt(5)^j in the code is the upper bound on omitted terms
\\ \\ noted in the sequence.  At the point of the check, variable HAf holds the
\\ \\ sum of terms 1 to j-1 inclusive.  The bound is
\\ \\
\\ \\     term(j) + term(j+1) + ...  <  1/sqrt(5)^j
\\ \\
\\ HAtermfZ(z) = 1/25 * ( 6*HAgrowf(      z) \
\\                      + 2*HAgrowf((4+I)*z) );
\\ HAtermf(j) = HAtermfZ(1/b^j);
\\ {
\\   for(j=1,50,
\\      my(rest = sum(i=0,100, term(j+i)));   \\ term(j) + ... onwards
\\      rest <= 1/sqrt(5)^j   || error());
\\ }
\\ 
\\ \\ "omitted" is kept as a rational by using it only when j is even, so
\\ \\ omitted = 1/5^(j/2).  PARI 2.13 up has comparisons of quads, so an exact
\\ \\ sqrt5 = quadgen(20) used appropriately would be fine, but hardly needed.
\\ \\
\\ \\ The loop stop test is as easy as checking whether omitted terms would
\\ \\ push HAf, after scaling by 10^len, would cross up into a different
\\ \\ integer.  If it does then the correct last digits are not yet known.
\\ \\ The actual digit values are not needed yet, only whether omitted is small
\\ \\ enough not to affect them.
\\ \\ 
\\ \\                + omitted
\\ \\            HAf --------->                HAf + omitted crosses up
\\ \\     |-------*------|-----*--------|      to a diferent integer, so
\\ \\    123            124            125     different (scaled) digits
\\ \\
\\ \\     |------->
\\ \\     frac(HAf)
\\ 
\\ 
\\ \\ The upper bound value 1/sqrt(5)^j as such is found by ignoring the
\\ \\ direction arg(1/b^j) and using only magitude abs(1/b^j) = 1/sqrt(5)^j.
\\ \\ 
\\ \\ termZ(z), as a function of abs(z) is bounded
\\ \\ 
\\ \\     termZ(z)  <=  1/25 * 26/sqrt5 * abs(z)
\\ \\
\\ \\ This follows by considering which regions of z get real(z) or 2*imag(z)
\\ \\ in HAgrowf(z), the way that those regions are related for z and (4+I)*z.
\\ \\ The maximum is attained when z is in direction 2+I (and any 90 degree
\\ \\ rotates i^d).
\\ {
\\   my(z=2+I);
\\   termZ(z) == 1/25 * 26   || error();
\\   \\ z=2+I has abs(z) == sqrt5 so that 25/sqrt5 * abs(z) == 26
\\ }
\\ 
\\ \\ The magnitudes of 1/sqrt5^j sum by a geometric series to 
\\ \\ 
\\ \\     1/sqrt5^j + 1/sqrt5^(j+1) + ...  =  (5+sqrt5)/4
\\ \\
\\ is_nearly_equal(x,y) = abs(x-y) < 1e-8;
\\ {
\\   for(j=1,20,
\\      is_nearly_equal(sum(i=0,100, 1/sqrt(5)^(j+i)),
\\                      (5+sqrt(5))/4 * 1/sqrt(5)^j        )  || error());
\\ }
\\ \\ For overall factor on 1/sqrt5^j of
\\ \\ 
\\ \\     1/25 * 26/sqrt5 * (5+sqrt5)/4
\\ \\     = 13/50 * (1+sqrt5)
\\ \\     = 0.84137767
\\ \\     < 1
\\ 
\\ \\ A slightly easier argument ignoring more of directions or factors could
\\ \\ reach a looser bound.  Or a slightly harder argument about the related
\\ \\ directions of a couple of 1/b^j and 1/b^(j=1) could reach a tighter
\\ \\ bound.  The plain 1/sqrt5^j form chosen is intended as easy for
\\ \\ understanding and use.
\\ 
\\ \\ The way HAgrowf and termZ vary with the direction of z can be seen in the
\\ \\ following plots.  Change to "if(1," to run them, since they're a bit big
\\ \\ and not directly related to A349009 digits.  The same with ploth() for a
\\ \\ graphical plot works too.
\\ \\
\\ \\ The maximum is for z in direction 2+I for both HAgrowf() and then
\\ \\ termZ(), which is angle theta = arg(2+I) = 0.463647... (A073000) radians.
\\ \\
\\ {
\\   if(0,
\\      plot(theta=0,Pi/2, my(z=exp(I*theta)); HAgrowf(z));
\\      plot(theta=0,Pi/2, my(z=exp(I*theta)); termZ(z));
\\      );
\\ }
