#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyR5Dragon;
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


printwrap("  % generated by tools/$FindBin::Script\n");

my $level = 2;

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my ($mx,$my) = ($x_hi/2, $y_hi/2);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}
$Printwrap = 2;
printwrap("\\draw [my line]");
$Printwrap += 2;
foreach my $line ($segments->lines) {
  my ($x1,$y1, $x2,$y2) = @$line;
  printwrap(" ($x1,$y1)--($x2,$y2)");
}
printwrap(";\n");

my $inertia = 0;
$Printwrap = 2;
my %seen;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);

  my ($rx,$ry) = ($x-$mx, $y-$my);  # radial dist from centre
  $inertia += $rx*$rx + $ry*$ry;

  next if $seen{"$x,$y"}++;

  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;
  my $dot = (@n_list == 2 ? 'my big dot' : 'my dot');
  printwrap(" \\node at ($x,$y) [$dot];");
}
printwrap_newline();

$inertia *= 2;
print "  \\coordinate (middle) at ($mx,$my);\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyInertia}{\\frac{$inertia}{2}}\n";
print "% GP-Test  IV($level)==$inertia/2\n";

print "  % end generated\n";
exit 0;
