\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

read("r5dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
z_mod_2(z) = (real(z)+imag(z))%2;

z_mod_b_list = [    0    , 1, 1+  I,     I,   2*I, \
                1+2*I, 2+2*I, 2+3*I, 1+3*I, 1+4*I ];
z_mod_b(z) =
{
  for(i=1,5,
     my(r=z_mod_b_list[i]);
     if(z_is_integer((z-r)/b),return(i-1)));
  error("oops, z_mod_b() not found");
}

z_mod_2b(z) = [0,3,4,1,2,5,8,9,6,7][((3*real(z) + imag(z)) % 10) + 1];
check_exact(vector(5,a,a--; z_mod_b(digitpoint(a))), \
            vector(5,a,a--; a), "z_mod_b vs digitpoint");
check_exact(vector(5,a,a--; z_mod_b(digitpoint(a) + (1+I)*b^2)), \
            vector(5,a,a--; a));

check_exact(vector(#z_mod_b_list,i, z_mod_2b(z_mod_b_list[i])), \
            vector(#z_mod_b_list,i,i--; i));

check_exact(matrix(11,11,x,y,x-=6;y-=6; vector(#z_mod_b_list,i, z_mod_2b(z_mod_b_list[i] + (x+y*I)*2*b))), \
             matrix(11,11,x,y,x-=6;y-=6; vector(#z_mod_b_list,i,i--; i)));

\\
\\        1+4*I -- 2+4*I
\\          |
\\        1+3*I -- 2+3*I
\\                   |
\\ 2*I -- 1+2*I -- 2+2*I
\\  |
\\  I ---- 1+I
\\          |
\\  0 ----- 1
\\

{
  forstep(y=10,0,-1,
         for(x=0,10,
            my(z=x+I*y);
            print1(" ",z_mod_b(z)));
         print());
}
         
{
  forstep(y=10,0,-1,
         for(x=0,10,
            my(z=x+I*y);
            printf("%3d",z_mod_2b(z)));
         print());
}

r5dragon_unpoint(z,s) = {
  my(debug=1);
  z_is_integer(z)
    || error("r5dragon_unpoint() should be integer z, got ",z);
  s==1 || s==-1
    || error("r5dragon_unpoint() should be s=+/-1, got ",s);
  if(z==0,return([0,0]));
  my(v=List([]));  \\ digits low to high
  if(debug,print("r5dragon_unpoint() z="z"  s="s));
  while(#v<6,  \\ ! (z==0 || z==-I) &&
    my(a=z_mod_2b(if(s<0,I*z,z)));
    my(delta=if(a>=5, -I*digitpoint(a-5), digitpoint(a)));
    if(debug,print("at z="z"  s="s"  digit="a" delta="delta" to "z-delta));
    z -= delta;
    z = z/b;
    if(s==-1,z*=-I);
    if(a==3||a==4, s = -s);
    \\ if(z_is_odd(z),s=-s);
    if(! z_is_integer(z), error("oops, z=",z));
    listput(v,a%5);
    );
  if(debug,print("end z="z" s="s" digits high to low "Vecrev(v)));
  v=Vecrev(v);
  if(z==0 && s==1,   [0, fromdigits(v,5)],
     z==0 && s==-1,  [0, fromdigits(v,5)],
     z==I        ,   [1, fromdigits(v,5)],
     z==1 && s==-1, \\ v=apply(a->if(a==4,3, a==3,4, a),v);
                    [3, 5^#v-fromdigits(v,5)],
     error("oops, r5dragon_unpoint() end z=",z," s=",s));
}

\\ point = b^0 * digitpoint(a0) * I^dir(ak-1..a1)
\\ digitpoint(a0) * I^dir(..a1) == z mod b
\\ have d=I^dir(..a0) =I^dir(..a1) * I^dir(a0)
\\ digitpoint(a0) * d / I^dir(a0) == z mod b
\\ digitpoint(a0) / I^dir(a0) == z/d mod b
check_exact(vector(5,a,a--; digitpoint(a) / I^dir(a)), \
            [0, -I, -1-I, 1, 2*I])
check_exact(apply(z_mod_b,[0, -I, -1-I, 1, 2*I]), \
            [0, 2, 3, 1, 4])

z_to_low_digit(z) = [0, 3, 2, 1, 4][(real(z)+2*imag(z))%5+1];
check_exact(apply(z_to_low_digit,[0, -I, -1-I, 1, 2*I]), \
            [0,1,2,3,4])

check_exact(apply(z_to_low_digit,[0, 1+I, I, 1, 2*I]), \
            [0,1,2,3,4])

\\        2*I
\\         
\\         0   1
\\  -1-I  -I
\\

r5dragon_unpoint(z,s) = {
  my(debug=0);
  z_is_integer(z)
    || error("r5dragon_unpoint() should be integer z, got ",z);
  s==1 || s==-1
    || error("r5dragon_unpoint() should be s=+/-1, got ",s);
  if(z==0,return([0,0]));
  my(v=List([]));  \\ digits low to high
  if(debug,print("r5dragon_unpoint() z="z"  s="s));
  my(d=s*I^z_mod_2(z));
  while(#v<6 && ! (z==0 || (z==I&&d==-I) || (z==-I&&d==I)),
    my(old_z=z,old_d=d);
    my(a=z_to_low_digit(z/d));
    d /= I^dir(a);
    my(delta=d*digitpoint(a));
    z -= delta;
    if(debug,print("at z="old_z" d="old_d
                   "  digit="a" new d="d" delta="delta" to "z));
    z = z/b;
    if(! z_is_integer(z), error("oops, z=",z));
    listput(v,a%5);
    );
  if(debug,print("end z="z" d="d" digits high to low "Vecrev(v)));
  v=Vecrev(v);
  if(z==0 && d==1,   [0, fromdigits(v,5)],
     z==I && d==-I,  [1, 5^#v-fromdigits(v,5)],
     z==0 && d==-1,  [2, fromdigits(v,5)],
     z==-I && d==I,  [3, 5^#v-fromdigits(v,5)],
     error("oops, r5dragon_unpoint() end z=",z," d=",d));
}

{
  for(n=1,20,
     print(r5dragon_unpoint(r5dragon_point(n),1)," ",
           r5dragon_unpoint(r5dragon_point(n),-1)));
}
quit

print()
r5dragon_unpoint(1,1)
r5dragon_unpoint(1,-1)
r5dragon_unpoint(-1,1)
r5dragon_unpoint(-1,-1)

print()
r5dragon_unpoint(1+I,1)
r5dragon_unpoint(1+I,-1)
r5dragon_unpoint(-1-I,1)
r5dragon_unpoint(-1-I,-1)

print()
{
  for(n=1,100,
     my(z=r5dragon_point(n));
     for(arm=0,3,
        my(z=z*I^arm);
        my(s=if(arm==0||arm==2,1,-1));
        \\ if(n%2,,s=-s);
        my(an=r5dragon_unpoint(z,s));
        if(!(an[1]==arm && an[2]==n),
           print("wrong n="n" arm="arm" z="z" s="s"  got "an);
           quit)));
}
