#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'xy_rotate_minus90','rect_corners_sort';
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $level = 6;

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\begin{scope}[rotate={-atan2($y_hi,$x_hi)},scale={1/sqrt(5)^$level}]\n";

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

$Printwrap = 2;
printwrap("\\fill foreach \\p in {");
$Printwrap += 2;
my $RQ = 0;
my %seen;
foreach my $n ($n_lo .. $n_hi-1) {
  $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my ($x2,$y2) = ($x1+$dx+$rx, $y1+$dy+$ry);
  ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
  next if $seen{"$x1,$y1"}++;
  printwrap("($x1,$y1),");
  $RQ++;
}
printwrap("}");
printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");

print "  \\end{scope}\n";
print "% GP-Test  RQ($level) == $RQ\n";
print "  % end generated\n";
exit 0;










# use MyLineSegments;
# print "  % generated by tools/$FindBin::Script\n";
# my $level = 6;
#
# my $path = Math::PlanePath::R5DragonCurve->new;
# my ($n_lo, $n_hi) = $path->level_to_n_range($level);
# my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
#
# print "  \\begin{scope}[rotate={-atan2($y_hi,$x_hi)},scale={1/sqrt(5)^$level}]\n";
#
# my $segments = MyLineSegments->new;
# foreach my $n ($n_lo .. $n_hi-1) {
#   $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
#   my ($x,$y) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   $segments->add($x,$y, $x2,$y2);
# }
#
# $Printwrap = 2;
# printwrap("\\fill ");
# $Printwrap = 4;
# $segments->printwrap;
# my $count = $segments->lines_joined_count_points;
# printwrap(";");
# printwrap(" % $count points\n");
#
# print "  \\end{scope}\n";
# print "  % end generated\n";
# exit 0;
