#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'Hwpf_box_points';
use MyPrintwrap ':all';


my $level = 2;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";

{
  my $points = Hwpf_box_points();
  print "  my Hwpf box pic/.pic={\n";
  $Printwrap = 4;
  printwrap("\\draw ");
  $Printwrap += 2;
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    $x = sprintf '%.8g', $x;
    $y = sprintf '%.8g', $y;
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap("${join}cycle;\n");
  print "  },\n";
}
{
  print "  my Hwpf k$level hull pic/.pic={\n";
  my $points = Hwpf_box_points();
  my $ex = 1;
  my $ey = 0;
  foreach (1 .. $level) {
    ($points, $ex,$ey) = MyR5Dragon::hull_expand($points, $ex,$ey, unsorted=>1);
  }
  my $z_end = Math::Complex->new($ex,$ey);
  my @z_points = map { Math::Complex->new(@$_) } @$points;
  @z_points = map { $_ / $z_end } @z_points;

  # print scalar(@$points)," points, end $ex,$ey\n";
  $Printwrap = 4;
  printwrap("\\draw ");
  $Printwrap += 2;
  my $join = '';
  foreach my $z (@z_points) {
    my $x = $z->Re;
    my $y = $z->Im;
    $x = sprintf '%.8g', $x;
    $y = sprintf '%.8g', $y;
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap("${join}cycle;");
  printwrap(" % ",scalar(@$points)," points\n");
  print "  },\n";
}
print "  % end generated\n";
exit 0;
