#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';
use Math::BigRat;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $n_middle = $n_hi / 2;
my ($x_middle,$y_middle) = $path->n_to_xy($n_middle);
{
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  $x_middle == $x_hi/2 or die;
  $y_middle == $y_hi/2 or die;
}

print "  % n_middle = $n_middle\n";
print "  \\coordinate (start) at (0,0);\n";
print "  \\coordinate (end) at ($x_middle*2,$y_middle*2);\n";

my $segments = MyLineSegments->new;
my $n = $n_lo;
# $n = int($n_middle-6);
for (; $n < $n_middle; $n++) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy(min($n+1,$n_middle));
  $x1 *= 2;
  $y1 *= 2;
  $x2 *= 2;
  $y2 *= 2;
  $segments->add($x1,$y1, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
