#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Graph::Maker::QuartetTree;

use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';

my $max_level = 4;
if (@ARGV) { $max_level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";

# return $x,$y multiplied by 2+i
# GP-Test  (2+I)*(x+I*y) == (2*x-y) + (x+2*y)*I
sub mul {
  my ($x,$y) = @_;
  return (2*$x -   $y,
          $x   + 2*$y);
}

my $ex = 1;
my $ey = 0;

foreach my $level (0 .. $max_level) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$level)]\n");

  my $graph = Graph::Maker->new('quartet_tree',
                                level => $level,
                                # vertex_names => 'xy-straight',
                                undirected=>1);

  my $segments = MyLineSegments->new;
  foreach my $edge ($graph->edges) {
    my ($from,$to) = @$edge;
    $segments->add((split /,/, $from),
                   (split /,/, $to));
  }
  $Printwrap = 4;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");

  $Printwrap = 4;
  printwrap("\\node (start-$level) at (0,0) [my dot];\n");
  printwrap("\\node (end-$level) at ($ex,$ey) [my dot];\n");

  if (0) {
    my %seen = ('0,0'=>1, "$ex,$ey"=>1);;
    printwrap("\\path foreach \\p in {");
    $Printwrap += 2;
    foreach my $xy ($graph->vertices) {
      if (! $seen{$xy}) { printwrap("($xy),"); }
    }
    printwrap("}");
    printwrap(" { \\p node[my dot] };\n");
  }

  my $count = $segments->lines_joined_count_points;
  my $num_vertices = $graph->vertices;
  print "    % $num_vertices vertices, lines drawn with $count points\n";

  $Printwrap = 2;
  print "  \\end{scope}\n";

  ($ex,$ey) = mul($ex,$ey);
}
print "  % end generated\n";
exit 0;
