#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use List::Util 'min','max';
use Math::PlanePath::R5DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';


my $max_k = 5;
if (@ARGV) { $max_k = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_k\n";
my $path = Math::PlanePath::R5DragonCurve->new;

print "% GP-DEFINE  GJN_samples_by_path = { 1/2 * [\n";
my @x_total;
my @y_total;
foreach my $k (1 .. $max_k) {
  my ($n_lo, $n_mid) = $path->level_to_n_range($k);
  my $n_hi = 2*$n_mid;

  my $x_total = 0;
  my $y_total = 0;
  my %seen;
  my $count = 0;
 N: foreach my $n ($n_lo .. $n_mid-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my $mx = $x;
    my $my = $y;
    my $above = 0;
    my $below = 1;
    foreach (1 .. 3) {
      $x += $dx;
      $y += $dy;
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);

      if ($x <= $mx && $y <= $my) {
        $mx = $x;
        $my = $y;
      }

      my $n = $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy) // next N;
      $n < $n_hi or next N;
      if ($n >= $n_mid) {
        $above++;
      } else {
        $below++;
      }
    }
    next unless $above;
    next if $seen{"$mx,$my"}++;

    $x_total += 2*$mx + 1;
    $y_total += 2*$my + 1;
    $count++;
  }
  my $J = J($k);
  $count == $J or die "k=$k  count=$count J=$J";

  my $comma = ($k == $max_k ? '' : ',');
  print "% GP-DEFINE     ($x_total) + ($y_total)*I$comma   /* k=$k */\n";
  $x_total[$k] = $x_total;
  $y_total[$k] = $y_total;
}
print "% GP-DEFINE  ]};\n";
print "% end generated\n";


if (0) {
  # for formula sample values
  print "  % generated by tools/$FindBin::Script\n";
  foreach my $k (1 .. min(5,$max_k)) {
    my $x_total = $x_total[$k];
    my $y_total = $y_total[$k];
    if ($x_total % 2 == 0 && $y_total % 2 == 0) {
      $x_total /= 2;
      $y_total /= 2;
      printf "  %d%+d i,\n", $x_total, $y_total;
    } else {
      printf "  \\tfrac{%d%+d i}{2},\n", $x_total, $y_total;
    }
  }
  print "  % end generated\n";

  print "  % generated by tools/$FindBin::Script\n";
  foreach my $k (1 .. min(5,$max_k)) {
    my $x_total = $x_total[$k];
    my $y_total = $y_total[$k];
    my $denominator = 2 * J($k);
    # if ($x_total % 2 == 0 && $y_total % 2 == 0) {
    #   $x_total /= 2;
    #   $y_total /= 2;
    #   $denominator /= 2;
    # }
    if ($denominator == 1) {
      printf "  %d%+d i,\n", $x_total, $y_total;
    } else {
      printf "  \\tfrac{%d%+d i}{$denominator},\n", $x_total, $y_total;
    }
  }
  print "  % end generated\n";
}

exit 0;
